@@ -155,16 +155,12 @@ let gen_deconstructible gen print eq = GenDeconstr (gen,print,eq)
 
 let qcheck_nat64_small = QCheck.(map Int64.of_int small_nat)
 
-let print_char c   = Printf.sprintf "%C" c
-let print_string s = Printf.sprintf "%S" s
-let print_bytes b  = print_string (Bytes.to_string b)
-
 let bytes_small_printable = QCheck.bytes_small_of QCheck.Gen.printable
 
 let unit =           GenDeconstr (QCheck.unit,           QCheck.Print.unit, (=))
 let bool =           GenDeconstr (QCheck.bool,           QCheck.Print.bool, (=))
-let char =           GenDeconstr (QCheck.char,           print_char,        (=))
-let char_printable = GenDeconstr (QCheck.printable_char, print_char,        (=))
+let char =           GenDeconstr (QCheck.char,           QCheck.Print.char, (=))
+let char_printable = GenDeconstr (QCheck.printable_char, QCheck.Print.char, (=))
 let nat_small =      GenDeconstr (QCheck.small_nat,      QCheck.Print.int,  (=))
 let int =            GenDeconstr (QCheck.int,            QCheck.Print.int,  (=))
 let int_small =      GenDeconstr (QCheck.small_int,      QCheck.Print.int,  (=))
@@ -173,13 +169,13 @@ let int_bound b =    GenDeconstr (QCheck.int_bound b,    QCheck.Print.int,  (=))
 let int32 =          GenDeconstr (QCheck.int32,          Int32.to_string,   Int32.equal)
 let int64 =          GenDeconstr (QCheck.int64,          Int64.to_string,   Int64.equal)
 let nat64_small =    GenDeconstr (qcheck_nat64_small,    Int64.to_string,   Int64.equal)
-let float =          GenDeconstr (QCheck.float,          Float.to_string,   Float.equal)
-let string =         GenDeconstr (QCheck.string,         print_string,      String.equal)
-let string_small =   GenDeconstr (QCheck.small_string,   print_string,      String.equal)
-let string_small_printable = GenDeconstr (QCheck.small_printable_string,   print_string,      String.equal)
-let bytes =          GenDeconstr (QCheck.bytes,          print_bytes,       Bytes.equal)
-let bytes_small =    GenDeconstr (QCheck.bytes_small,    print_bytes,       Bytes.equal)
-let bytes_small_printable = GenDeconstr (bytes_small_printable,   print_bytes,      Bytes.equal)
+let float =          GenDeconstr (QCheck.float,          QCheck.Print.float,  Float.equal)
+let string =         GenDeconstr (QCheck.string,         QCheck.Print.string, String.equal)
+let string_small =   GenDeconstr (QCheck.small_string,   QCheck.Print.string, String.equal)
+let string_small_printable = GenDeconstr (QCheck.small_printable_string, QCheck.Print.string, String.equal)
+let bytes =          GenDeconstr (QCheck.bytes,          QCheck.Print.bytes, Bytes.equal)
+let bytes_small =    GenDeconstr (QCheck.bytes_small,    QCheck.Print.bytes, Bytes.equal)
+let bytes_small_printable = GenDeconstr (bytes_small_printable, QCheck.Print.bytes, Bytes.equal)
 
 let option : type a c s. ?ratio:float -> (a, c, s, combinable) ty -> (a option, c, s, combinable) ty =
   fun ?ratio ty ->
@@ -195,12 +191,24 @@ let list : type a c s. (a, c, s, combinable) ty -> (a list, c, s, combinable) ty
     | GenDeconstr (arb, print, eq) -> GenDeconstr (QCheck.list arb, QCheck.Print.list print, List.equal eq)
     | Deconstr (print, eq) -> Deconstr (QCheck.Print.list print, List.equal eq)
 
+let list_small : type a c s. (a, c, s, combinable) ty -> (a list, c, s, combinable) ty =
+  fun ty -> match ty with
+    | Gen (arb, print) -> Gen (QCheck.small_list arb, QCheck.Print.list print)
+    | GenDeconstr (arb, print, eq) -> GenDeconstr (QCheck.small_list arb, QCheck.Print.list print, List.equal eq)
+    | Deconstr (print, eq) -> Deconstr (QCheck.Print.list print, List.equal eq)
+
 let array : type a c s. (a, c, s, combinable) ty -> (a array, c, s, combinable) ty =
   fun ty -> match ty with
     | Gen (arb, print) -> Gen (QCheck.array arb, QCheck.Print.array print)
     | GenDeconstr (arb, print, eq) -> GenDeconstr (QCheck.array arb, QCheck.Print.array print, Array.for_all2 eq)
     | Deconstr (print, eq) -> Deconstr (QCheck.Print.array print, Array.for_all2 eq)
 
+let array_small : type a c s. (a, c, s, combinable) ty -> (a array, c, s, combinable) ty =
+  fun ty -> match ty with
+    | Gen (arb, print) -> Gen (QCheck.array_of_size QCheck.Gen.small_nat arb, QCheck.Print.array print)
+    | GenDeconstr (arb, print, eq) -> GenDeconstr (QCheck.array_of_size QCheck.Gen.small_nat arb, QCheck.Print.array print, Array.for_all2 eq)
+    | Deconstr (print, eq) -> Deconstr (QCheck.Print.array print, Array.for_all2 eq)
+
 let seq_iteri f s =
   let (_:int) = Seq.fold_left (fun i x -> f i x; i + 1) 0 s in ()
 
@@ -214,11 +222,11 @@ let print_seq pp s =
   Buffer.add_char b '>';
   Buffer.contents b
 
-let arb_seq a =
+let arb_seq size_gen a =
   let open QCheck in
   let print = match a.print with None -> None | Some ap -> Some (print_seq ap) in
   let shrink s = Iter.map List.to_seq (Shrink.list ?shrink:a.shrink (List.of_seq s)) in
-  let gen = Gen.map List.to_seq (Gen.list a.gen) in
+  let gen = Gen.map List.to_seq (Gen.list_size size_gen a.gen) in
   QCheck.make ?print ~shrink gen
 
 let rec seq_equal eq s1 s2 =
@@ -230,8 +238,14 @@ let rec seq_equal eq s1 s2 =
 
 let seq : type a c s. (a, c, s, combinable) ty -> (a Seq.t, c, s, combinable) ty =
   fun ty -> match ty with
-    | Gen (arb, print) -> Gen (arb_seq arb, print_seq print)
-    | GenDeconstr (arb, print, eq) -> GenDeconstr (arb_seq arb, print_seq print, seq_equal eq)
+    | Gen (arb, print) -> Gen (arb_seq QCheck.Gen.nat arb, print_seq print)
+    | GenDeconstr (arb, print, eq) -> GenDeconstr (arb_seq QCheck.Gen.nat arb, print_seq print, seq_equal eq)
+    | Deconstr (print, eq) -> Deconstr (print_seq print, seq_equal eq)
+
+let seq_small : type a c s. (a, c, s, combinable) ty -> (a Seq.t, c, s, combinable) ty =
+  fun ty -> match ty with
+    | Gen (arb, print) -> Gen (arb_seq QCheck.Gen.small_nat arb, print_seq print)
+    | GenDeconstr (arb, print, eq) -> GenDeconstr (arb_seq QCheck.Gen.small_nat arb, print_seq print, seq_equal eq)
     | Deconstr (print, eq) -> Deconstr (print_seq print, seq_equal eq)
 
 let state = State
